@@ -4,7 +4,7 @@ from decimal import Decimal
 import re
 
 from lxml import etree
-from stream_unzip import stream_unzip
+from stream_unzip import UnzipValueError, stream_unzip
 
 
 def stream_read_ods(ods_chunks, chunk_size=65536):
@@ -38,13 +38,33 @@ def stream_read_ods(ods_chunks, chunk_size=65536):
 
         return FileLikeObj()
 
-    def get_member_file(single_file_name, unzipped_files):
-        for name, size, chunks in unzipped_files:
-            if name != single_file_name:
+    def validate_mimetype_and_get_content(unzipped_files):
+        correct_mimetype = b'application/vnd.oasis.opendocument.spreadsheet'
+
+        for i, (name, size, chunks) in enumerate(unzipped_files):
+            if i == 0 and name != b'mimetype':
+                raise MissingMIMETypeError()
+
+            if i == 0 and name == b'mimetype':
+                mimetype = b''
+                chunks_it = iter(chunks)
+                while len(mimetype) < len(correct_mimetype):
+                    try:
+                        mimetype += next(chunks_it)
+                    except StopIteration:
+                        break
+                if mimetype != correct_mimetype:
+                    raise IncorrectMIMETypeError(mimetype.decode("utf-8"))
+
+            if name != b'content.xml':
                 for chunk in chunks:
                     pass
                 continue
+
             yield from chunks
+            break
+        else:
+            raise MissingContentXMLError()
 
     def get_sheets_and_rows(parsed_xml):
 
@@ -84,12 +104,12 @@ def stream_read_ods(ods_chunks, chunk_size=65536):
             if value_type != 'string':
                 return \
                     None if value_type is None else \
-                    parse_boolean(cell_element.attrib[f'{ns_office}boolean-value']) if value_type == 'boolean' else \
-                    parse_currency(cell_element, cell_element.attrib[f'{ns_office}value']) if value_type == 'currency' else \
-                    parse_date(cell_element.attrib[f'{ns_office}date-value']) if value_type == 'date' else \
-                    parse_float(cell_element.attrib[f'{ns_office}value']) if value_type == 'float' else \
-                    parse_percentage(cell_element.attrib[f'{ns_office}value']) if value_type == 'percentage' else \
-                    parse_time(cell_element.attrib[f'{ns_office}time-value']) if value_type == 'time' else \
+                    parse_boolean(cell_element) if value_type == 'boolean' else \
+                    parse_currency(cell_element) if value_type == 'currency' else \
+                    parse_date(cell_element) if value_type == 'date' else \
+                    parse_float(cell_element) if value_type == 'float' else \
+                    parse_percentage(cell_element) if value_type == 'percentage' else \
+                    parse_time(cell_element) if value_type == 'time' else \
                     value_error(value_type)
 
             # Strings can be from an attribute...
@@ -106,29 +126,31 @@ def stream_read_ods(ods_chunks, chunk_size=65536):
         def value_error(message):
             raise ValueError(message)
 
-        def parse_boolean(value):
+        def parse_boolean(cell_element):
+            value = cell_element.attrib[f'{ns_office}boolean-value']
             return \
                 True if value == 'true' else \
                 False if value == 'false' else \
                 value_error(value)
 
-        def parse_currency(cell_element, value):
-            return Currency(value, code=cell_element.attrib.get(f'{ns_office}currency'))
+        def parse_currency(cell_element):
+            return Currency(cell_element.attrib[f'{ns_office}value'], code=cell_element.attrib.get(f'{ns_office}currency'))
 
-        def parse_date(value):
+        def parse_date(cell_element):
+            value = cell_element.attrib[f'{ns_office}date-value']
             try:
                 return date.fromisoformat(value)
             except ValueError:
                 return datetime.fromisoformat(value)
 
-        def parse_float(value):
-            return Decimal(value)
+        def parse_float(cell_element):
+            return Decimal(cell_element.attrib[f'{ns_office}value'])
 
-        def parse_percentage(value):
-            return Percentage(value)
+        def parse_percentage(cell_element):
+            return Percentage(cell_element.attrib[f'{ns_office}value'])
 
-        def parse_time(value):
-            time_dict = time_regex.match(value).groupdict(0)
+        def parse_time(cell_element):
+            time_dict = time_regex.match(cell_element.attrib[f'{ns_office}time-value']).groupdict(0)
             return Time(
                 time_dict.get('sign') or '+',
                 int(time_dict.get('years', '0')),
@@ -157,15 +179,22 @@ def stream_read_ods(ods_chunks, chunk_size=65536):
             # Starting a table
             if event == 'start' and f'{ns_table}table' == element.tag:
                 sheet_name = element.attrib[f'{ns_table}name']
-                yield sheet_name, table_rows(parsed_xml_it)
+                rows = table_rows(parsed_xml_it)
+                yield sheet_name, rows
+                for _ in rows:
+                    raise UnfinishedIterationError()
 
             clear_mem(event, element)
 
     unzipped_member_files = stream_unzip(ods_chunks, chunk_size=chunk_size)
-    content_xml_chunks = get_member_file(b'content.xml', unzipped_member_files)
+    content_xml_chunks = validate_mimetype_and_get_content(unzipped_member_files)
     content_xml_file_like_obj = to_file_like_obj(content_xml_chunks)
     content_xml_parsed = etree.iterparse(content_xml_file_like_obj, events=('start', 'end'))
-    yield from get_sheets_and_rows(content_xml_parsed)
+
+    try:
+        yield from get_sheets_and_rows(content_xml_parsed)
+    except UnzipValueError as e:
+        raise UnzipError() from e
 
 
 def simple_table(rows, skip_rows=0):
@@ -186,8 +215,6 @@ def simple_table(rows, skip_rows=0):
             yield row
 
     for i, row in enumerate(rows):
-        if i < skip_rows:
-            pass
         if i == skip_rows:
             header_row = up_to_first_none(row)
             return header_row, remaining_rows(width=len(header_row))
@@ -211,3 +238,35 @@ class Currency(Decimal):
 
 
 Time = namedtuple('Time', ('sign', 'years', 'months', 'days', 'hours', 'minutes', 'seconds'), defaults=('+', 0, 0, 0, 0, Decimal('0')))
+
+
+class StreamReadODSError(Exception):
+    pass
+
+
+class InvalidOperationError(StreamReadODSError):
+    pass
+
+
+class UnfinishedIterationError(InvalidOperationError):
+    pass
+
+
+class InvalidODSFileError(StreamReadODSError, ValueError):
+    pass
+
+
+class UnzipError(InvalidODSFileError):
+    pass
+
+
+class MissingMIMETypeError(InvalidODSFileError):
+    pass
+
+
+class IncorrectMIMETypeError(InvalidODSFileError):
+    pass
+
+
+class MissingContentXMLError(InvalidODSFileError):
+    pass
